'use strict';

const _ = require(`lodash`);
const Bluebird = require(`bluebird`);
const conventionalGithubReleaser = Bluebird.promisify(require(`conventional-github-releaser`));
const debug = require(`debug`)(`semantic-release-github-releaser`);
const getPkgRepo = require(`get-pkg-repo`);

module.exports = githubReleaser;

function githubReleaser(config) {
  if (!_.has(config, `pkg`)) {
    return Bluebird.reject(new Error('This plugin, `semantic-release-github-releaser`, was not ' +
      'passed the contents of your package\'s `package.json` file. Please contact the user of ' +
      'this plugin and request that they pass the contents of `package.json` to the plugin.'));
  }

  let repoUrl;
  try {
    repoUrl = getPkgRepo(config.pkg);
    debug(`parsed repository URL using the 'get-pkg-repo' package - %O`, repoUrl);
  } catch (err) {
    return Bluebird.reject(err);
  }

  if (repoUrl.domain === undefined) {
    debug(`no domain found in parsed repository URL while using the 'get-pkg-repo' package`);
    return Bluebird.reject(new Error(`Unable to parse the repository URL.`));
  }

  if (typeof config.options.scmToken !== `string` || config.options.scmToken.length === 0) {
    debug(`no SCM token provided for GitHub`);
    return Bluebird.reject(new Error(`No SCM token provided for GitHub.`));
  }

  const auth = {
    url: `${config.options.insecureApi ? `http` : `https`}://api.${repoUrl.domain}/`,
    token: config.options.scmToken,
  };

  debug(`config - %O`, auth);

  return conventionalGithubReleaser(auth, {
    preset: config.options.preset || `angular`,
  }, {
    owner: repoUrl.user,
    repository: repoUrl.project,
  }, {
    merges: null,
  })
    .then(() => {
      debug(`successfully generated GitHub release`);

      return true;
    });
}
